(*********************************************
 * Talmi Moshinsky Transformation 
 * Mathematica package
 * Yang Li, 10/05/2011, <leeyoung@iastate.edu>
 * updated: 01/20/2017 
 *********************************************)

 BeginPackage["TalmiMoshinsky`"]

TalmiMoshinsky::usage =
"TalmiMoshinsky[N, M, n, m, n1, m1, n2, m2, tan(delta)] computes the Talmi-Moshinsky\n coefficients for 2D harmonic oscillator functions."

TMC::usage="TMC[N, M, n, m, n1, m1, n2, m2, tan(delta)] or TMC[N, M, n, m, n1, m1, n2, m2, x1, x2] calculates the 2D Talmi-Moshinsky coefficient with generalized binomial and multinomial coefficients. tan(delta)=sqrt(x2/x1)."

Begin["TalmiMoshinsky`Private`"]

Print["* * * TalmiMoshinsky.m * * *"];
Print["Yang Li <leeyoung@iastate.edu>, Oct. 05, 2011, updated: Jan. 20, 2017"];
Print["This package provides the calculation of Talmi-Moshinsky transformation brackets. \n\nFunctions:\nTalmiMoshinsky[N, M, n, m, n1, m1, n2, m2, Tan[delta]] \ncomputes the Talmi-Moshinsky coefficients for 2D harmonic \noscillator functions."];

Print["\nTMC[N, M, n, m, n1, m1, n2, m2, Tan[delta]] \nand TMC[N, M, n, m, n1, m1, n2, m2, x2, x1] \nare aliases for TalmiMoshinsky, with Tan[delta] = Sqrt[x2/x1]."];
Print["* * * * * * * * * * * * * * *"];

Clear[TMC, TalmiMoshinsky];

TMC[N1_Integer, M1_Integer, N2_Integer, M2_Integer, n1_Integer, m1_Integer, n2_Integer, m2_Integer, t_] := TalmiMoshinsky[N1, M1, N2, M2, n1, m1, n2, m2, t] 

TMC[N1_Integer, M1_Integer, N2_Integer, M2_Integer, n1_Integer, m1_Integer, n2_Integer, m2_Integer, x1_, x2_] := TalmiMoshinsky[N1, M1, N2, M2, n1, m1, n2, m2, Sqrt[x2/x1]] 

TalmiMoshinsky[N1_Integer, M1_Integer, N2_Integer, M2_Integer, n1_Integer, m1_Integer, n2_Integer, m2_Integer, t_] :=  
  Module[
	  {s, c, r, E1p, E1m, E2p, E2m, e1p, e1m, pref, part1, part2},

      If[2*N1+Abs[M1]+2*N2+Abs[M2]==2*n1+Abs[m1]+2*n2+Abs[m2] && 
	     M1+M2==m1+m2, 

	     s=t/Sqrt[1+t^2];
	     c=1/Sqrt[1+t^2];
	     r=-1/t^2;

	     E1p = N1 + (Abs[M1]+M1)/2;
	     E1m = N1 + (Abs[M1]-M1)/2;
	     E2p = N2 + (Abs[M2]+M2)/2;
	     E2m = N2 + (Abs[M2]-M2)/2;
	     e1p = n1 + (Abs[m1]+m1)/2;
	     e1m = n1 + (Abs[m1]-m1)/2;

         pref = (-1)^(N1+N2+n1+n2+M2+m1) * t^(2*n1+Abs[m1]) * s^(2*N1+Abs[M1]) * c^(2*N2+Abs[M2]) 
	          * Sqrt[n1!/N1!*n2!/N2!*(n1+Abs[m1])!/(N1+Abs[M1])!*(n2+Abs[m2])!/(N2+Abs[M2])!];
	     part1 = Sum[ r^i * Binomial[E1m, i] * Binomial[E2m, e1m-i], {i, Max[0, e1m-E2m], Min[e1m,E1m]}];
	     part2 = Sum[ r^i * Binomial[E1p, i] * Binomial[E2p, e1p-i], {i, Max[0, e1p-E2p], Min[e1p,E1p]}];
	  pref*part1*part2
	  ,
	  0
	  ]
	]

End[ ]

EndPackage[ ]
